%This is the main programme. It is assumed that the folder in which this
%file is located is the current folder at start.

%Graphs are saved in the "Graphs" subfolder.
%Sensitivity tables are saved to "Tables" subfolder.
%Products of Dynare solution are saved to "Dynare" subfolder.
 
%1 TECHNICAL PRELIMINARIES
close all
clear all
clc

addpath(pwd)

%2 SET PARAMETERS

%structural parameters
NZLB = (1.02)^(1/4)-1; %the lower bound on (net) interest rates, in quarterly terms
pi_star=(1.02)^(1/4); %gross steady state inflation, in quarterly terms
rho = 0.8; %serial correlation of natural interest rate shock
e0 = -0.05; %shock to natural interest rates, in quarterly terms
sigma=0.157; %intertemporal elasticity of substitution, Woodford (1999)
beta=0.99; %households' discount factor (impatience), Woodford (1999)
potential_gdp_growth=0.02; %matching long-term real per capita gdp growth in the US economy
one_minus_alpha=0.7; %Cobb-Douglas exponent on labour, Smets Wouters (2002)
phi=0.25; %elasticity of labour supply, Smets Wouters (2003)
epsilon=3; %elasticity of substitution between labour varieties, Smets Wouters (2002)
theta=find_theta(0.024,beta,one_minus_alpha,phi,epsilon,sigma); %Calvo probability, set so as to match Woodford's kappa=0.024 under the rest given 

%compile the parameter set
%parameter_set=[sigma,beta,beta1,beta2,kappa1,kappa2,lambda,potential_gdp_growth,pi_star-1,theta,one_minus_alpha,phi,epsilon];
parameter_set=[sigma,beta,potential_gdp_growth,pi_star-1,theta,one_minus_alpha,phi,epsilon];

%run simulations under commitment and discretion
bl_commitment=run_commitment(NZLB,rho,e0,parameter_set);
bl_discretion=run_discretion(NZLB,rho,e0,parameter_set);

%create break-even graphs between discretion and commitment
beac=zeros(99,4);
for jj=0:99
	rho_be=jj/100;
	results1=solve_NZLB_com(NZLB,rho_be,e0,parameter_set,0);
	beac(jj+1,1)=sum(results1(:,9));
    results2=solve_NZLB_dis(NZLB,rho_be,e0,parameter_set,0);
	beac(jj+1,3)=sum(results2(:,7));
end
beac(:,2)=bl_discretion*ones(100,1);
beac(:,4)=bl_commitment*ones(100,1);
%18: baseline: ZLB discretion
figure(18)
h1 = plot(linspace(0.7,0.99,30)',beac(71:100,1),'--k');
hold on;
h2 = plot(linspace(0.7,0.99,30)',beac(71:100,2),'-k');
legend('loss under commitment and PLB (different rhos)','loss under discretion and ZLB (rho=0.8)')
title('Loss under commitment and PLB (different rho) versus baseline loss under discretion');
xa_def = [0.7 0.7 0.8 0.8 0.7]; ya_def = [00 0.03 0.03 0 0];
h5 = fill(xa_def,ya_def,[0.9 0.9 0.9], 'LineStyle', 'none');
set(gca, 'Children', [h1 h2 h5])
saveas(18,'Graphs/18_loss_com_base_dis.jpg','jpg');
descr={'rho','loss under commitment and PLB (different rhos)','loss under discretion and ZLB (rho=0.8)'};
xlswrite('Graphs/data.xlsx',descr,18,'B1');
xlswrite('Graphs/data.xlsx',[linspace(0,0.99,100)',beac(1:100,1),beac(1:100,2)],18,'B2');
%19: baseline: ZLB commitment
figure(19)
h1 = plot(linspace(0.2,0.7,51)',beac(21:71,3),'--k');
hold on;
h2 = plot(linspace(0.2,0.7,51)',beac(21:71,4),'-k');
legend('loss under discretion and PLB (different rhos)','loss under commitment and ZLB (rho=0.8)')
title('Loss under discretion and PLB (different rho) versus baseline loss under commitment');
xa_def = [0.5 0.5 0.7 0.7 0.5]; ya_def = [0 0.007 0.007 0 0];
h5 = fill(xa_def,ya_def,[0.9 0.9 0.9], 'LineStyle', 'none');
set(gca, 'Children', [h1 h2 h5])
saveas(19,'Graphs/19_loss_dis_base_com.jpg','jpg');
descr={'rho','loss under discretion and PLB (different rhos)','loss under commitment and ZLB (rho=0.8)'};
xlswrite('Graphs/data.xlsx',descr,19,'B1');
xlswrite('Graphs/data.xlsx',[linspace(0,0.99,100)',beac(1:100,3),beac(1:100,4)],19,'B2');


%plot com vs dis graphs
load('crossgraphs.mat');
load('crossgraphs2.mat');
nn=min(size(y_zlb_com,1),size(y_zlb_dis,1));
%OUTPUT GAP
figure(20)
plot(0:(nn-1),y_zlb_com(1:nn),'--k',0:(nn-1),y_nzlb_dis(1:nn),'-k')
legend('zero lower bound, commitment','lower bound=0.02/4, discretion','Location','South')
title('Output gap under different lower bounds and commitment vs discretion (1)')
%axis([0 (max_length-1)  -0.4 0.1]);
saveas(20,'Graphs/20_output_ZLBcom_PLBdis.jpg','jpg');
descr={'zero lower bound, commitment','lower bound=0.02/4, discretion'};
xlswrite('Graphs/data.xlsx',descr,20,'B1');
xlswrite('Graphs/data.xlsx',[y_zlb_com(1:nn),y_nzlb_dis(1:nn)],20,'B2');

figure(21)
plot(0:(nn-1),y_zlb_dis(1:nn),'--k',0:(nn-1),y_nzlb_com(1:nn),'-k')
legend('zero lower bound, discretion','lower bound=0.02/4, commitment','Location','South')
title('Output gap under different lower bounds and commitment vs discretion (2)')
saveas(21,'Graphs/21_output_ZLBdis_PLBcom.jpg','jpg');
descr={'zero lower bound, discretion','lower bound=0.02/4, commitment'};
xlswrite('Graphs/data.xlsx',descr,21,'B1');
xlswrite('Graphs/data.xlsx',[y_zlb_dis(1:nn),y_nzlb_com(1:nn)],21,'B2');

figure(22)
plot(0:(nn-1),pi_zlb_com(1:nn),'--k',0:(nn-1),pi_nzlb_dis(1:nn),'-k')
legend('zero lower bound, commitment','lower bound=0.02/4, discretion','Location','South')
title('Inflation rate under different lower bounds and commitment vs discretion (1)')
saveas(22,'Graphs/22_inflation_ZLBcom_PLBdis.jpg','jpg');
descr={'zero lower bound, commitment','lower bound=0.02/4, discretion'};
xlswrite('Graphs/data.xlsx',descr,22,'B1');
xlswrite('Graphs/data.xlsx',[pi_zlb_com(1:nn),pi_nzlb_dis(1:nn)],22,'B2');

figure(23)
plot(0:(nn-1),pi_zlb_dis(1:nn),'--k',0:(nn-1),pi_nzlb_com(1:nn),'-k')
legend('zero lower bound, discretion','lower bound=0.02/4, commitment','Location','South')
title('Inflation rate under different lower bounds and commitment vs discretion (2)')
saveas(23,'Graphs/23_inflation_ZLBdis_PLBcom.jpg','jpg');
descr={'zero lower bound, discretion','lower bound=0.02/4, commitment'};
xlswrite('Graphs/data.xlsx',descr,23,'B1');
xlswrite('Graphs/data.xlsx',[pi_zlb_dis(1:nn),pi_nzlb_com(1:nn)],23,'B2');

figure(24)
plot(0:(nn-1),i_zlb_com(1:nn),'--k',0:(nn-1),i_nzlb_dis(1:nn),'-k')
legend('zero lower bound, commitment','lower bound=0.02/4, discretion','Location','South')
title('Nominal interest rate under different lower bounds and commitment vs discretion (1)')
saveas(24,'Graphs/24_NIR_ZLBcom_PLBdis.jpg','jpg');
descr={'zero lower bound, commitment','lower bound=0.02/4, discretion'};
xlswrite('Graphs/data.xlsx',descr,24,'B1');
xlswrite('Graphs/data.xlsx',[i_zlb_com(1:nn),i_nzlb_dis(1:nn)],24,'B2');

figure(25)
plot(0:(nn-1),i_zlb_dis(1:nn),'--k',0:(nn-1),i_nzlb_com(1:nn),'-k')
legend('zero lower bound, discretion','lower bound=0.02/4, commitment','Location','South')
title('Nominal interest rate under different lower bounds and commitment vs discretion (2)')
saveas(25,'Graphs/25_NIR_ZLBdis_PLBcom.jpg','jpg');
descr={'zero lower bound, discretion','lower bound=0.02/4, commitment'};
xlswrite('Graphs/data.xlsx',descr,25,'B1');
xlswrite('Graphs/data.xlsx',[i_zlb_dis(1:nn),i_nzlb_com(1:nn)],25,'B2');

load('crossgraphs3.mat');
load('crossgraphs4.mat');

figure(26)
plot(rho_grid,equiv_dis_plb_vsCom,'--k',rho_grid,e0_line,'-k')
xlabel('rho')
ylabel('e1')
legend('equvalence of PLB (discretion, given rho and e1) to ZLB (commitment, baseline rho and e1)','baseline e1','Location','South')
axis([0.5 0.9 -0.06 -0.01])
title('Loss equivalence: ZLB (commitment) vs PLB (discretion) under different rho and e1');
saveas(26,'Graphs/26_equivalence_ZLBcom_PLBdis.jpg','jpg');
descr={'rho','equvalence of PLB (discretion, given rho and e1) to ZLB (commitment, baseline rho and e1)','baseline e1'};
xlswrite('Graphs/data.xlsx',descr,26,'B1');
xlswrite('Graphs/data.xlsx',[rho_grid,equiv_dis_plb_vsCom,e0_line],26,'B2');

figure(27)
plot(rho_grid,equiv_com_plb_vsDis,'--k',rho_grid,e0_line,'-k')
xlabel('rho')
ylabel('e1')
legend('equivalence of PLB (commitment, given rho and e1) to ZLB (discretion, baseline rho and e1)','baseline e1','Location','South')
axis([0.5 0.9 -0.15 -0.03])
title('Loss equivalence: ZLB (discretion) vs PLB (commitment) under different rho and e1');
saveas(27,'Graphs/27_equivalence_ZLBdis_PLBcom.jpg','jpg');
descr={'rho','equivalence of PLB (commitment, given rho and e1) to ZLB (discretion, baseline rho and e1)','baseline e1'};
xlswrite('Graphs/data.xlsx',descr,27,'B1');
xlswrite('Graphs/data.xlsx',[rho_grid,equiv_com_plb_vsDis,e0_line],27,'B2');

beac=zeros(99,1);
parameter_set_alt1=[parameter_set(1,1),parameter_set(1,2),0.04,(1.04)^(1/4)-1,0.9,parameter_set(1,6),0.1,10];
results1z=solve_NZLB_dis(0,rho,e0,parameter_set_alt1,0);
loss1z=sum(results1z(:,7));

for jj=0:98
	rho_be=jj/100;
    results1=solve_NZLB_com(NZLB,rho_be,e0,parameter_set_alt1,0);
	beac(jj+1,1)=sum(results1(:,9));

end

figure(30)
h1 = plot(linspace(0,0.98,99)',beac(:,1),'--k');
hold on
%h2 = plot(linspace(0,0.98,99)',beac(:,2)/loss2z,'-k');
h2 = plot(linspace(0,0.98,99)',loss1z*ones(99,1),'-k');
legend('PLB under commitment, growth=0.04, inf.target=0.04, theta=0.9, phi=0.1, epsilon=10','ZLB under discretion, baseline rho and the same calibration');
title('Loss under positive lower bound under different parameters, as a function of rho');
xa_def = [0.5 0.5 0.8 0.8 0.5]; ya_def = [0.0002 0.045 0.045 0.0002 0.0002];
h5 = fill(xa_def,ya_def,[0.9 0.9 0.9], 'LineStyle', 'none');
set(gca, 'Children', [h1 h2 h5])
%axis([0 (max_length-1)  -0.3 0.1]);
saveas(30,'Graphs/30_BE_AC_comdis.jpg','jpg');
descr={'rho','loss function value set 1, PLB commitment vs ZLB discretion'};
xlswrite('Graphs/data.xlsx',descr,30,'B1');
xlswrite('Graphs/data.xlsx',[linspace(0,0.98,99)',beac(:,1),loss1z*ones(99,1)],30,'B2');
