function [results]=solve_NZLB_dis(NZLB,rho,e0,parameter_set,showplot,baseline_loss)

% rho - serial correlation of the natural interest rate shock e0
% e0 - initial magnitude of the shock
% NZLB - where to stop cutting nominal interest rate
% showplot - 0 if plots to be shown, 1 if not
% baseline_loss: number for the purpose of presentation, useless when showplot=0

%1 DEFINE DEEP PARAMETERS

sigma=parameter_set(1,1);
beta=parameter_set(1,2);
potential_gdp_growth=parameter_set(1,3);
pi_star=parameter_set(1,4);
theta=parameter_set(1,5);
one_minus_alpha=parameter_set(1,6);
phi=parameter_set(1,7);
epsilon=parameter_set(1,8);

[NKPC_PI,~]=positive_inf_steady(beta,theta,one_minus_alpha,phi,epsilon,sigma,pi_star+1);
beta1=NKPC_PI(1,1);
beta2=NKPC_PI(1,2);
kappa1=NKPC_PI(1,3);
kappa2=NKPC_PI(1,4);
aux_const=(pi_star+1)^(epsilon-1)*((pi_star+1)^(1-epsilon)-theta)^(-epsilon/(1-epsilon))*(1-theta)^(1/(1-epsilon));
lambda = ((1-beta*theta)*aux_const^2)/(1-theta-aux_const^2)*(phi+sigma)/epsilon;

%2 COMPUTE SOME REDUCED-FORM PARAMETERS

nat_int_rate=sigma*potential_gdp_growth/4+(1-beta)*beta;

%3 SIMULATE UNTIL T+1
   
	%3a GENERATE THE SHOCK VECTOR
    horizon1=500;
	shock=zeros(horizon1,1);
	shock(1,1)=e0;
	shock(1,1)=shock(1,1)+nat_int_rate;
	for ii=1:horizon1-1;
		shock(ii+1,1)=rho*(shock(ii,1)-nat_int_rate)+nat_int_rate;
	end;

    %3b FIND THE TIME OF LB BINDING
    T=0;
    for ii=1:ceil(horizon1/2);
       if shock(ii,1)+pi_star<NZLB;
          T=T+1;
       else
          break;
       end;
    end; 
    
	%3c SOLVE FOR PERIODS 1...T,T+1
    nn=3*T+3;
    ENDO=zeros(nn,nn);

    %IS curve
    for ii=1:T;
       ENDO(ii,ii)=1;
       ENDO(ii,ii+1)=-1;
       ENDO(ii,ii+T+2)=-sigma^(-1);
       ENDO(ii,ii+T+2+T+1)=sigma^(-1);
    end;
    
    %Phillips curve
    for ii=T+1:2*T;
       ENDO(ii,ii-T)=-kappa1;
       ENDO(ii,ii-T+1)=-kappa2;
       ENDO(ii,ii-T+T+1)=1;
       ENDO(ii,ii-T+T+1+1)=-beta1;
       ENDO(ii,ii-T+T+2+1)=-beta2;
    end;
    
    %positive lower bound definition
    for ii=2*T+1:3*T;
       ENDO(ii,ii-2*T+2*T+2+1)=1;
    end;

    %initial and terminal conditions
    ENDO(3*T+1,T+1)=1;
    ENDO(3*T+2,2*T+2)=1;
    ENDO(3*T+3,2*T+3)=1;

    %exogenous side
	EXO=zeros(nn,1);
    for ii=1:T;
       EXO(ii,1)=sigma^(-1)*(shock(ii,1)+pi_star);
    end;
	EXO(2*T+1:3*T,1) = NZLB*ones(T,1);

	SOLVED=ENDO\EXO;

	%3d GENERATE THE RESULT CONTAINERS

    horizon2 = 3*T+15;
	Y=zeros(horizon2,1);	%output gap
	PI=zeros(horizon2,1);  %inflation
    NIR=zeros(horizon2,1);   %nominal interest rate
    
	for ii=2:T+2;
	   Y(ii,1)=SOLVED(ii-1,1);
	end;

	for ii=2:T+2;
	   PI(ii,1)=SOLVED(ii-1+T+1,1)+pi_star;
	end;

	for ii=2:T+1;
	   NIR(ii,1)=SOLVED(ii-1+2*T+3,1);
	end;

%4 SIMULATE AFTER T+1

for ii=T+1:horizon2;
   NIR(ii,1)=shock(ii,1)+pi_star;
   PI(ii,1)=pi_star;
end;

%5 FILL IN THE REST OF THE CONTAINERS

%nominal interest rate
NIR(1,1)=nat_int_rate+pi_star;

%real interest rate
RIR=zeros(horizon2,1);
RIR(1,1)=nat_int_rate;
for ii=2:horizon2-1;
   RIR(ii,1)=NIR(ii,1)-PI(ii+1,1);
end;
RIR(horizon2,1)=NaN;

%natural interest rate
RN=zeros(3*T+1,1);
RN(1,1)=nat_int_rate;
for ii=2:horizon2;
   RN(ii,1)=shock(ii-1,1);
end;

%5 SUMMARIZE AND EVALUATE

discount=ones(size(Y,1),1);
for ii=3:size(Y,1)
    discount(ii,1)=beta*discount(ii-1,1);
end
loss = discount.*((Y.^2)*lambda+(PI-pi_star*ones(size(PI,1),1)).^2);
results = [linspace(0,horizon2-1,horizon2)' Y PI NIR RIR RN loss];

%6 DISPLAY THE RESULTS

if showplot==1

	annot_text_1=['Period 0: steady state' 10 'Period 1: shock occurs' 10 'Period ' num2str(T+1) ': lower bound no more binding'];
	annot_text_2=['Period 0: steady state' 10 'Period 1: shock occurs' 10 'Period ' num2str(T+1) ': lower bound no more binding'];

	figure(9)
	plot(0:horizon2-1,NIR(:,1),'-k',0:horizon2-1,RN(:,1),':k',0:horizon2-1,RIR(:,1),'--k',0:horizon2-1,PI(:,1),'-.k')
	legend('nominal interest rate','natural interest rate','real interest rate (ex ante)','inflation rate')
	title('Interest rates and inflation - discretion')
	axis([0 horizon2 -0.06 0.05])
	text(5,-0.02,annot_text_1);
	saveas(9,'Graphs/9_IRF_rates_pi_dis.jpg','jpg');
    descr={'nominal interest rate','natural interest rate','real interest rate (ex ante)','inflation rate'};
    xlswrite('Graphs/data.xlsx',descr,9,'B1');
    xlswrite('Graphs/data.xlsx',[NIR(:,1),RN(:,1),RIR(:,1),PI(:,1)],9,'B2');

	figure(10)
	plot(0:horizon2-1,Y,'-k')
	title('Output gap - discretion')
	legend('output gap');
	axis([0 3*T -1.3 0.1])
	text(7,-0.6,annot_text_2);
	saveas(10,'Graphs/10_IRF_output_pi_dis.jpg','jpg');
    descr={'output gap'};
    xlswrite('Graphs/data.xlsx',descr,10,'B1');
    xlswrite('Graphs/data.xlsx',Y,10,'B2');
	
    display(['Discretion: loss function value: ' num2str(sum(loss)) ' as compared to ' num2str(baseline_loss) ' under ZLB']);
    
end


